#!/bin/bash
# This is dxr3config by Wicky
# for requests and suggestions send a pn to Wicky @ www.vdrportal.de


##### tmppath for dxr3.tmp ######################
# Path where tmp files are stored, the dxr3 load and unload scripts are placed 
# and dxr3config looks for dxr3config.m2v
# Attention:
# If TMPPATH=/usr/bin,/usr/sbin,/bin,/sbin dxr3config changes this path to /usr/share/misc/dxr3
# In this case you have to store dxr3config.m2v in /usr/share/misc/dxr3
TMPPATH=$(pwd)

# In which file are the module configurations stored, only used if modify="true"
# If modify="false" then the file $(pwd)/modprobe.conf.dxr3 will be generated
moduleFile="/etc/modprobe.conf"

#In which directroy are module configurations placed ?
modprobed="/etc/modprobe.d"

# Dont change this preset, use the skript parameter -modify instead!!
modify="false"

##### path to dxr3config.m2v ######
# setting for linvdr-dxr3.deb :
#TEST_JPEG_PATH=/usr/share/dxr3deb
TEST_JPEG_PATH=$(pwd)

version="0.2.0rc2"
title="dxr3config Version $version"
auto="false"
genScripts="false"

typeset -i paramSet

paramSet=0

advParam[1]="adv717x pixelport_16bit=1 pixelport_other_pal=1"
emParam[1]="em8300 dicom_fix=1 dicom_control=1 dicom_other_pal=1"

advParam[2]="adv717x pixelport_16bit=0 pixelport_other_pal=0"
emParam[2]="em8300 dicom_fix=1 dicom_control=1 dicom_other_pal=0"

advParam[3]="adv717x pixelport_16bit=0 pixelport_other_pal=1"
emParam[3]="em8300 dicom_fix=1 dicom_control=1 dicom_other_pal=1"

advParam[4]="adv717x pixelport_16bit=1 pixelport_other_pal=0"
emParam[4]="em8300 dicom_fix=0 dicom_control=0 dicom_other_pal=0"

advParam[5]=""
emParam[5]=""

advParam[6]=""
emParam[6]=""

advParam[7]=""
emParam[7]=""

advParam[8]=""
emParam[8]=""

advParam[9]=""
emParam[9]=""

advParam[10]="bt865"
emParam[10]="em8300 dicom_fix=1 dicom_control=1 dicom_other_pal=1 use_bt865=1"

advParam[11]="bt865"
emParam[11]="em8300 dicom_fix=1 dicom_control=1 dicom_other_pal=1 use_bt865=1 bt865_ucode_timeout=1"

advParam[12]="bt865"
emParam[12]="em8300 dicom_fix=1 dicom_control=1 dicom_other_pal=0 use_bt865=1 bt865_ucode_timeout=1"

advParam[13]="bt865"
emParam[13]="em8300 dicom_fix=0 dicom_control=0 dicom_other_pal=0 use_bt865=1 bt865_ucode_timeout=1"

advParam[14]=""
emParam[14]=""

advParam[15]=""
emParam[15]=""


redblueParam[2]="pixeldata_adjust_pal=0"
redblueParam[3]="pixeldata_adjust_pal=1"
redblueParam[4]="pixeldata_adjust_pal=2"
redblueParam[5]="pixeldata_adjust_pal=3"
redblueParam[6]="pixeldata_adjust_ntsc=0"
redblueParam[7]="pixeldata_adjust_ntsc=1"
redblueParam[8]="pixeldata_adjust_ntsc=2"
redblueParam[9]="pixeldata_adjust_ntsc=3"

adapterParam[1]="output_mode=comp+svideo"
adapterParam[2]="output_mode=comp+psvideo"
adapterParam[3]="output_mode=psvideo"
adapterParam[4]="output_mode=composvideo"
adapterParam[5]="output_mode=yuv"

adapter=""
redblue=""

eepromOut[2]="0000000 0000 0000 0000 0000 0000 0000 0000 0000 0000020 0000 0000 0000 0000 0000 0000 0000 0000 0000040 0000 0000 0000 0000 0000 0000 0000 0000 0000060 0000 0000 0000 0000 0000 0000 0000 0000 0000100 39b8 6099 5617 3301 dcbb 5f68 18cc 6b63 0000120 0000 0000 0000 0000 0000 0000 0000 0000 0000140 0000 0000 0000 0000 0000 0000 0000 0000 0000160 0000 0000 0000 0000 0000 0000 0000 0000 0000200 ffff ffff ffff ffff ffff ffff ffff ffff 0000220 ffff ffff ffff ffff ffff ffff ffff ffff 0000240 ffff ffff ffff ffff ffff ffff ffff ffff 0000260 ffff ffff ffff ffff ffff ffff ffff ffff 0000300 ffff ffff ffff ffff ffff ffff ffff ffff 0000320 ffff ffff ffff ffff ffff ffff ffff ffff 0000340 ffff ffff ffff ffff ffff ffff ffff ffff 0000360 ffff ffff ffff ffff ffff ffff ffff ffff 0000400"
eepromOut[2]=$(echo ${eepromOut[2]} | tr " " "x")

function usage ()
{
        echo ""
        echo "dxr3config $version"
        echo ""
        echo "dxr3config - a small tool to generate a dxr3 module configuration"
        echo ""
        echo "Options:"
        echo "        --auto           non-interactive mode"
        echo "        --chipParam      list dxr3configs adv717x/bt865 parameters"
        echo "        --adapterParam   list dxr3configs adapter parameters"
        echo "        --redblueParam   lists dxr3configs red/blue parameters"    
        echo "        --help           this message"
        echo ""
        echo "This options can only be used with --auto:"
        echo ""
        echo "        --modModprobe    modifies modprobe.conf"
        echo "                         a copy of your modprobe will be made."
        echo "        --genModprobe    copys a mod-utils compatible"
        echo "                         dxr3 configuration file in /etc/modprobe.d"
        echo "        --genScripts     generates load and unload scripts"
        echo ""
}

function printChipParam ()
{
        echo "adv717x and bt865 Parameters:"
        echo ""
        for i in 1 2 3 4;  do
                echo "Parameterset $i:         modprobe ${advParam[i]}"
                echo "Parameterset $i:         modprobe ${emParam[i]}"
                echo ""
        done
        for i in 10 11 12 13;  do
                echo "Parameterset $i:        modprobe ${advParam[i]}"
                echo "Parameterset $i:        modprobe ${emParam[i]}"
                echo ""
        done
}

function printAdapterParam ()
{
        echo "dxr3config adapter parameters:"
        for i in 1 2 3 4 5; do
                echo ""
                echo "Adapter parameter $i:        ${adapterParam[i]}"
        done
        echo ""
}

function printRedBlueParam ()
{
        echo "dxr3config red/blue parameters:"
        for i in 1 2 3 4 5 6 7 8 9; do
                echo ""
                echo "red/blue parameter $i:        ${redblueParam[i]}"
        done
        echo ""
}

function checkPwd ()
{
        function newTMP
        {
                mkdir /usr/share/misc/dxr3 >/dev/null 2>/dev/null;
                TMPPATH="/usr/share/misc/dxr3";
        }
        case $(which dxr3config.sh) in
                "/usr/bin/dxr3config.sh") newTMP;;
                "/usr/sbin/dxr3config.sh")newTMP;;
                "/bin/dxr3config.sh")     newTMP;;
                "/sbin/dxr3config.sh")    newTMP;;
        esac
}

# read script parameter
while [ $# -gt 0 ]; do
        case $1 in
                --auto)         auto="true";
                                shift;;
                --modModprobe)       modify="true"
                                shift;;
                --genScripts)   genScripts="true"
                                shift;;
                --genModprobe)  genModprobe="true"
                                shift;;
                --chipParam)    printChipParam;
                                exit;;
                --adapterParam) printAdapterParam;
                                exit;;
                --redblueParam) printRedBlueParam;
                                exit;;
                        *)      usage;
                                exit;;
        esac
done

function quit () 
{
        if [ -w $TMPPATH/load_dxr3.tmp ] ; then 
                rm $TMPPATH/load_dxr3.tmp;
        fi
        if [ -w $TMPPATH/playM2V.stop ]; then
                rm $TMPPATH/playM2V.stop
        fi
        rm -f $TMPPATH/dxr3.tmp
        rm -f $TMPPATH/playM2V.sh
        rm -f $TMPPATH/tmp.m2v
        rm -f $TMPPATH/dxr3error.log
        rm -f $TMPPATH/emptyModprobe.conf
        if [ "$auto" = "false" ];then
                 clear
        fi
        exit $1;
}


function checkVDRrunning ()
{
        vdrPID=`ps -C vdr -o "%p" | grep -vw PID | tac`
        if [ x"$vdrPID" != "x" ]; then
                if [ "$auto" = "true" ]; then
                        echo "";
                        echo "VDR is running. Stop it before running dxr3config."
                        echo "";
                        quit 2;
                else
                        msg "**** VDR is running ****\\nYou have to stop it before running dxr3config.";
                        quit 2;
                fi
        fi
}

function checkRoot ()
{
        if [ $(id -u) != "0" ]; then
                msg  "You are not loged in as user root.\\n **** dxr3config will not work ****\\nLogin as user root, before you use dxr3config.";
                quit 3;
        fi
}

function preCheck ()
{
        clear
        echo "Checking your dxr3 hard- and software..............."
        failedModule=""
        rmmodules
        # ********** check module loading **********
        for module in adv717x bt865 em8300; do
                modprobe -q -C $empty $module 2>/dev/null
                moduleLoad=$(grep -w "^${module%% *}" /proc/modules | awk '"$module" {print $3}')
                if [ $moduleLoad -le 2 ] 2> /dev/null ; then                        
                        printf "loading module $module \t \t \t \t \033[0;32mok\033[0m\\n";
                else
                        printf "loading module $module \t \t \t \t \033[0;31mfailed\033[0m\\n";
                        logger "dxr3config[$$]: Error: $module could not be loaded"
                fi
        done


        # ************ detecting hardware *************
        dxr3Rev=""
	dxr3Rev=$(lspci -n | grep -w 1105:8300 | awk '{print $NF}' | sed 's/(//g' | sed 's/)//g')
        if [ -n "$dxr3Rev" ]; then
                printf "detecting dxr3-hardware  \t \t \t \033[0;32mok\033[0m\\n"
                printf "dxr3 hardware revision  \t \t \t \033[0;32m$dxr3Rev\033[0m\\n"
                advUsed="1";
                advUsed=$(grep -w "^adv717x" /proc/modules | awk '"$module" {print $3}')
                if [ "$advUsed" = "1" ]; then
                        printf "detecting adv717x/bt865 \t \t \t \033[0;32madv717x\033[0m\\n"
                        chip="adv717x"
                else
                        printf "detecting adv717x/bt865 \t \t \t \033[0;32mbt865\033[0m\\n"
                        chip="bt865"
                fi

        else 
                printf "detecting dxr3-hardware  \t \t \t \033[0;31mNo dxr3 mpeg-decoder found\033[0m\\n"
                logger "dxr3config[$$]: Error: no dxr3 mpeg-decoder found"
        fi

        # ************ playing mpeg to get more Infos  *************
        startMpeg
        sleep 2
        setParamset
        kernelVersion=$(uname -r)
        echo "Card revision $cardRev"
        echo "Chip revision $chipRev"
        echo "videoEncoder $videoEncoder"
        echo "moduleVersion $moduleVersion"        
        # ********* check for em8300setup
        em8300setupPath=$(which em8300setup);
        noEm8300setup="0";
        if [ "$em8300setupPath" != "" ]; then
                printf "looking for em8300setup \t \t \033[0;32m $em8300setupPath \033[0m\\n";
                em8300setupOut=$(em8300setup | grep 'Microcode uploaded')
                if [ "$em8300setupOut" != "" ]; then
                        printf "looking for Microcode \t \t \t \t\033[0;32mfound\033[0m\\n";
                else
                        printf "looking for Microcode \t \t \t \t \033[0;31mnot found\033[0m\\n"
                        noEm8300setup="1";
                fi
        else
                printf "looking for em8300setup \t \033[0;31mnot present, may be it's not needed\033[0m\\n";                
                noEm8300setup="1";
        fi

        # *********** check if the devices exist ********************
        for dev in /dev/em8300-0 /dev/em8300_mv-0 /dev/em8300_ma-0 /dev/em8300_sp-0; do
                if [ -e  $dev ]; then
                        printf "detecting device $dev \t\t \033[0;32mok\033[0m\\n"
                else
                        printf "detecting device $dev \t\t \033[0;31mfailed\033[0m\\n"
                        logger "dxr3config[$$]: Error: $dev is not available"
                fi
        done
        stopMpeg
        sleep 2

        # ********** check if the modules can be removed  **********
        for module in em8300 adv717x bt865; do
                moduleRemove=""
                modprobe -q -C $empty -r $module >/dev/null 2>/dev/null
                moduleRemove=$(grep -w "^${module%% *}" /proc/modules)
                if [ -n "$moduleRemove" ] >/dev/null 2>/dev/null; then
                        printf "removing modul $module \t \t \t \t \033[0;31mfailed\033[0m\\n"
                        logger "dxr3config[$$]: $module could not be unloaded"
                else
                        printf "removing modul $module \t \t \t \t \033[0;32mok\033[0m\\n";                        
                fi
        done


        # ******** dxr3config.m2v available ??? ***********        
        if [ -f $TMPPATH/dxr3config.m2v ]; then
                printf "detecting dxr3config.m2v \t\t\t \033[0;32mok\033[0m\\n"
        else
                printf "detecting dxr3config.m2v....."
                printf "\033[0;31mNo dxr3config.m2v found in $TMPPATH\033[0m\\n"
        fi


        echo ""
        echo "Press [RETURN]"
        read;
}

function setParamset ()
# An mpgeg must be playing while calling setParamset !!!
{        
        cat /proc/em8300/0 > $TMPPATH/dxr3info.txt
        numCol=$(awk '{print NF}' $TMPPATH/dxr3info.txt)
        cardRev=$(grep 'Card revision' $TMPPATH/dxr3info.txt | awk '{print $NF}')
        chipRev=$(grep 'Chip revision' $TMPPATH/dxr3info.txt | awk '{print $NF}')
        videoEncoder=$(grep 'Video encoder:' $TMPPATH/dxr3info.txt | awk '{print $3}')
        videoEncoder=$(grep 'Video encoder:' $TMPPATH/dxr3info.txt | awk '{print $3}')
        moduleVersion=$(grep 'em8300 module version' $TMPPATH/dxr3info.txt | awk '{print $NF}')
        rm $TMPPATH/dxr3info.txt
        dxr3sum=$cardRev$chipRev$videoEncoder
        case $dxr3sum in
                12ADV7175)      paramSet=3;;
                22ADV7175)      paramSet=3;;
                22BT865)        paramSet=11;;
                22ADV7170)      getEEpromInfo;
                                case $eepromInfo in
                                        ${eepromOut[2]})        paramSet=2;;
                                        *)                      paramSet=3;;
                                esac;;
                *)              echo ""
                                echo "dxr3config could not determine the chip parameters :-( .......";
                                echo "Maybe you are not using the standard microcode";
                                echo "";
                                if [ $auto = "true" ]; then
                                        quit 6
                                fi;;
        esac
}

function getEEpromInfo ()
{
        modprobe eeprom       
        if test ! -d /sys/devices/platform ; then                        
                        echo "Error: noSysfs"
        else
                found=no
                for f in /sys/devices/platform/i2c-* ; do
                        if grep EM8300 $f/name >/dev/null 2>&1 ; then
                                for e in $f/*/eeprom ; do
                                        if test -f $e ; then
                                                found=yes
                                                eepromInfo=$(od -v -h $e) 
                                                eepromInfo=$(echo ${eepromInfo} | tr " " "x")
                                        fi
                                done
                        fi
                done
                if test "$found" != "yes" ; then
                        echo "Errror: dxr3-EEprom found"
                fi
        fi
}

function rmmodules ()
{
        moduleRemoveError=""
        moduleRemove=""
        for module in em8300 adv717x bt865; do  
                        moduleRemove=""
                        modprobe -q -r $module >/dev/null 2>/dev/null
                        moduleRemove=$(grep -w "^${module%% *}" /proc/modules)
                        if [ -n "$moduleRemove" ] >/dev/null 2>/dev/null; then
                                if [ "$auto" = "true" ]; then
                                        echo "ERROR: The dxr3-modules $module could not be removed."
                                        quit 5;
                                else
                                        msg "ERROR: The dxr3-modules $module could not be removed.\\n\\nIf this happens again, you should leave dxr3config.sh and try it after a reboot."
                                fi
                                logger "dxr3config[$$]: Error: The module $module could not be removed"
                                moduleRemoveError="1"
                        fi
        done
}


function loadModules ()
{
        echo "ok" > $TMPPATH/dxr3error.log
        if [ $paramSet != "0" ]; then
                echo "ok" > $TMPPATH/dxr3error.log
                modprobe -C $empty ${advParam[$paramSet]} $adapter $redblue 2>> $TMPPATH/dxr3error.log
                modprobe -C $empty ${emParam[$paramSet]} 2>> $TMPPATH/dxr3error.log
                if [ "$(cat $TMPPATH/dxr3error.log)" != "ok" ]; then
                        msg "An error occured while loading the dxr3-modules...";
                else
                        if [ "$noEm8300setup" = "0" ]; then
                                em8300setup -p -a -o >/dev/null 2>/dev/null;
                        fi
                fi
        else
                echo "ok" > $TMPPATH/dxr3error.log
                modprobe bt865 2>> $TMPPATH/dxr3error.log
                modprobe adv717x 2>> $TMPPATH/dxr3error.log
                modprobe em8300 2>> $TMPPATH/dxr3error.log
                if [ "$(cat $TMPPATH/dxr3error.log)" != "ok" ]; then
                        echo "An error occured while loading the dxr3-modules...";
                        quit 4;
                else
                        if [ "$noEm8300setup" = "0" ]; then
                                em8300setup -p -a -o >/dev/null 2>/dev/null;
                        fi
                fi
        fi
}



function save ()
{
        if [ "$auto" = "false" ]; then
                dialog --backtitle "$title" --menu "dxr3config can now modify your modprobe.conf or generate entries for your dxr3 and copy them in  /etc/modprobe.d/dxr3\\n \\nIf you are unsure, use option B or C." 0 0 0 \
                "A" "Modify the modprobe.conf" \
                "B" "Generate a file with entries for the modprobe.conf" \
                "C" "Don't generate or modify anything." 2> $TMPPATH/dxr3.tmp;
                saveOption=$(cat $TMPPATH/dxr3.tmp);
                case $saveOption in
                        A)      modify="true";
                                makeModprobe;
                                msg "This file has been modified: \\n\\n$moduleFile\\n\\nA copy of the old file has been placed in:\\n\\n$moduleFile.orig";;
                        B)      modify="false";
                                makeModprobe;
                                msg "This file has been generated or modifyed:\\n\\n$moduleFile\\n\\nA copy of the old file has been placed in:\\n\\n$TMPPATH";;
                        *)      ;;
                esac
                dialog --backtitle "$title" --menu "dxr3config can now generate the two skripts load_dxr3 and unload_dxr3. You can use this skripts to load the dxr3 modules." 0 0 0 \
                "A" "Generate load_dxr3 and unload_dxr3" \
                "B" "Don't generate anything." 2> $TMPPATH/dxr3.tmp;                
                saveOption=$(cat $TMPPATH/dxr3.tmp);
                case $saveOption in
                        A)      makeScripts;
                                msg "The files load_dxr3 and unload_dxr3 has been save to:\\n\\n$TMPPATH";;
                        *)      ;;
                esac
        else
                if [ "$genModprobe" = "true" ]; then
                        makeModprobe;
                fi
                if [ "$modify" = "true" ]; then
                        makeModprobe;
                fi
                if [ "$genScripts" = "true" ]; then
                        makeScripts;
                fi
        fi
}

function msg ()
{
        dialog --backtitle "$title" --msgbox "$1" 0 0;
}

function makeScripts ()
{
        echo "#!/bin/bash" > $TMPPATH/load_dxr3;
        echo "# This file was generated by dxr3config Version: $version" >> $TMPPATH/load_dxr3;
        echo "# You can use it to load the modules for your dxr3 card" >> $TMPPATH/load_dxr3;
        echo "# You are using parameter set $paramSet" >> $TMPPATH/load_dxr3; 
        echo "modprobe i2c-core" >> $TMPPATH/load_dxr3
        echo "modprobe i2c-algo-bit" >> $TMPPATH/load_dxr3
        echo "modprobe ${advParam[$paramSet]} $adapter $redblue" >> $TMPPATH/load_dxr3;
        echo "modprobe ${emParam[$paramSet]}" >> $TMPPATH/load_dxr3;
        chmod 775 $TMPPATH/load_dxr3;

        echo "#!/bin/sh" > $TMPPATH/unload_dxr3;
        echo "#This file was generated by dxr3config $version" >> $TMPPATH/unload_dxr3;
        echo "#You can use it to unload the modules for your dxr3 card" >> $TMPPATH/unload_dxr3;
        echo "" >> $TMPPATH/unload_dxr3;
        echo "#fuser -k /dev/em8300-0" >> $TMPPATH/unload_dxr3;
        echo "#fuser -k /dev/em8300_mv-0" >> $TMPPATH/unload_dxr3;
        echo "#fuser -k /dev/em8300_ma-0" >> $TMPPATH/unload_dxr3;
        echo "modprobe -r em8300" >> $TMPPATH/unload_dxr3;
        echo "modprobe -r adv717x" >> $TMPPATH/unload_dxr3;
        echo "#modprobe -r i2c-algo-bit" >> $TMPPATH/unload_dxr3;
        chmod 775 $TMPPATH/unload_dxr3;
}

function makeModprobe ()
{
                if [ $paramSet -gt 9 ]; then
                        chip="bt865"
                else
                        chip="adv717x"
                fi
                if [ $modify = "false" ]; then
                        # Attention: moduleFile is here a directory !
                        moduleFile=$modprobed                        
                        if [ ! -d $moduleFile ]; then
                                mkdir $moduleFile >/dev/null 2>/dev/null
                        fi
                        # Attention: Now moduleFile gets now a file !
                        moduleFile="$moduleFile/dxr3"                        
                        mv -f $moduleFile $TMPPATH/dxr3.old >/dev/null 2>/dev/null
                else
                        cp $moduleFile $moduleFile.orig >/dev/null 2>/dev/null
                fi
                if [ -r $moduleFile ]; then
                        sed -i '/^[[:blank:]]*#/!{/^.*\(adv717x\|bt865\|em8300\)/s/\(.*\)/#\1/}' $moduleFile
                fi
                echo "#adv717x bt865 em8300 Module configuration for a dxr3-Card generated by dxr3config $version" >> $moduleFile
                echo "#chip-parameterset: $paramSet" >> $moduleFile
                echo "install em8300 /sbin/modprobe $chip && /sbin/modprobe --ignore-install em8300" >> $moduleFile
                echo "remove em8300 /sbin/modprobe -r --ignore-remove em8300 && /sbin/modprobe -r $chip" >> $moduleFile
                echo "options ${advParam[$paramSet]} $adapter $redblue" >> $moduleFile
                echo "options ${emParam[$paramSet]}"  >> $moduleFile
}

function menuParam ()
{
dialog --backtitle "$title" --menu "Option $paramSuggestion should fit for your dxr3-card." 0 0 0 \
        "1" "only for adv717x" \
        "2" "only for adv717x" \
        "3" "only for adv717x" \
        "4" "only for adv717x" \
        "5" "only for adv717x (unused)" \
        "6" "only for adv717x (unused)" \
        "7" "only for adv717x (unused)" \
        "8" "only for adv717x (unused)" \
        "9" "only for adv717x (unused)" \
        "10" "only for bt865"   \
        "11" "only for bt865"   \
        "12" "only for bt865"   \
        "13" "only for bt865"   \
        "14" "only for bt865 (unused)"   \
        "15" "only for bt865 (unused)" 2> $TMPPATH/dxr3.tmp;
        paramOption=$(cat $TMPPATH/dxr3.tmp);
        if [ $paramOption != "0" ]; then
                paramSet=$paramOption;
        fi;
}

function menuAdapter ()
{
        dialog --backtitle "$title" --menu "Try one of these options:" 0 0 0 \
        "1" "For svideo-cables or the SigmaDesings adapter (default)" \
        "2" "For svideo to SCART or composite adapters (only composite output)" \
        "3" "For some svideo to composite adapters (only composite output)" \
        "4" "For some svideo to composite adapters (only composite output)" \
        "5" "For special YUV-adapters (Be carefull!)" 2> $TMPPATH/dxr3.tmp;
        adaptertmp=$(cat $TMPPATH/dxr3.tmp)
        if [ -z $adaptertmp ]; then
                loop3=0;
        else
                adapter=${adapterParam[$adaptertmp]};
        fi
}


function menuRedBlue ()
{
        dialog --backtitle "$title" --menu "Try one of this options:" 0 0 0 \
        "2" "only for PAL (try this first)" \
        "3" "only for PAL (default for PAL)" \
        "4" "only for PAL" \
        "5" "only for pal" \
        "6" "only for NTSC" \
        "7" "only for NTSC (default for NTSC)" \
        "8" "only for NTSC" \
        "9" "only for NTSC" 2> $TMPPATH/dxr3.tmp;
        redbluetmp=$(cat $TMPPATH/dxr3.tmp);
        if [ -z "$redbluetmp" ]; then
                loop3="0";
        else
                redblue=${redblueParam[$redbluetmp]};
        fi
}

function genplayer ()
{
        echo "while [ 0 -eq 0 ]; do" > $TMPPATH/playM2V.sh
        echo "if [ -f $TMPPATH/playM2V.stop ]; then" >> $TMPPATH/playM2V.sh
        echo "rm $TMPPATH/playM2V.stop" >> $TMPPATH/playM2V.sh
        echo "exit" >> $TMPPATH/playM2V.sh
        echo "fi" >> $TMPPATH/playM2V.sh
        echo "cat $TMPPATH/tmp.m2v >/dev/em8300_mv-0 2>/dev/null" >> $TMPPATH/playM2V.sh        
        echo "done" >> $TMPPATH/playM2V.sh
        chmod 775 $TMPPATH/playM2V.sh
}

function startMpeg ()
{
        if [ -e  /dev/em8300_mv-0 ]; then
                $TMPPATH//playM2V.sh >/dev/null 2>/dev/null &
        else
                missingDev
        fi
}

function stopMpeg ()
{
                touch $TMPPATH/playM2V.stop
                catID=""
                catID=$(ps ax | grep -v grep | grep "cat $TMPPATH/tmp.m2v" | awk '{print $1}')                
                if [ -n "$catID" ]; then
                        kill -SIGKILL $catID;
                fi
                sleep 2
}

function missingDev ()
{
        dialog --backtitle "$title" --msgbox "The device /dev/em8300_mv-0 is not available.\\ndxr3config can not play the test image\\n\\n Try it again after a reboot." 9 50;
        quit 1;
}

function showTestPicture ()
{
        dialog --backtitle "$title" --msgbox "Now the modules will be loaded and a test picture will be shown on your TV-screen...." 8 54
        dialog --backtitle "$title" --infobox "starting, please wait......." 0 0;
        loadModules
        sleep 2
        startMpeg
        sleep 2
        msg "Stop playing?"
        dialog --backtitle "$title" --infobox "stopping, please wait......." 0 0
        stopMpeg
        rmmodules
}


function menuTestResult ()
{
        dialog --backtitle "$title" --menu "Choose an option:" 0 0 0 \
        "BW" "The image was only/nearly Black&White (adapter settings)" \
        "RB" "RED and BLUE was inverted" \
        "P" "The colors looked very strange" \
        "EXIT" "Exit without saving" \
        "SAVE" "Save and exit" 2> $TMPPATH/dxr3.tmp;
        testresult=$(cat $TMPPATH/dxr3.tmp);
}

function menuTestResultbt865 ()
{
        dialog --backtitle "$title" --menu "Choose an option:" 0 0 0 \
        "P" "I want to try an other parameterset" \
        "EXIT" "Exit without saving" \
        "SAVE" "Save and exit" 2> $TMPPATH/dxr3.tmp;
        testresult=$(cat $TMPPATH/dxr3.tmp);
}

function genM2V ()
{
        cat $TMPPATH/dxr3config.m2v > $TMPPATH/tmp.m2v;
        typeset -i i;
        i=1;
        while [ $i -le 500 ]; do
                cat $TMPPATH/dxr3config.m2v >> $TMPPATH/tmp.m2v
                i=$((i+1))
        done
}

checkPwd
empty=$TMPPATH/emptyModprobe.conf
echo "#empty" > $empty

modprobe i2c-core
modprobe i2c-algo-bit
checkVDRrunning
checkRoot
genM2V
genplayer


if [ "$auto" = "true" ]; then
        echo "determining dxr3-module parameters, please wait...";
        loadModules;
        startMpeg;
        sleep 2;
        setParamset;
        stopMpeg;
        save;
        quit 0;
else
        debugInfo
        preCheck
        paramSuggestion=$paramSet
        loop1="1";
        while [ x$loop1 == x"1" ]; do
                menuParam
                loop2="0";
                while [ $loop2 != "4"  ]; do
                        if [ $paramOption -gt 0 ]; then
                                showTestPicture
                        fi
                        loop3="0";
                        while [ x$loop3 == x"0" ]; do
                                loop3="1";
                                if [ $paramOption -gt 9 ]; then
                                        menuTestResultbt865;
                                else
                                        menuTestResult;
                                fi
                                case $testresult in
                                        BW)     menuAdapter;;   
                                        RB)     menuRedBlue;;
                                        P)      loop2="4";
                                                loop1="1";;
                                        EXIT)   loop2="4";
                                                loop1="2";;
                                        SAVE)   save;
                                                loop2="4";
                                                loop1="2";;
                                        *)      loop2="4";
                                                loop1="2";;
                                esac
                        done
                done
        done
        msg "Good Bye";
        quit 0
fi
